@@ -7,27 +7,13 @@ implementing the [`Tokens`] trait (for example `&str` and `&[T]`).
 */
 use std::borrow::Borrow;
 
-pub trait Tokens<Item>: Iterator<Item=Item> {
+pub trait Tokens: Iterator {
+
     /// An object which can be used to reset the token stream 
     /// to some position.
-    type CheckPoint;
+    type Location: TokenLocation + PartialEq;
 
-    /// The current index into the token stream.
-    ///
-    /// # Example
-    ///
-    /// ```rust
-    /// use yap::{ Tokens, IntoTokens };
-    ///
-    /// let mut s = "abcde".into_tokens();
-    /// s.next().unwrap();
-    /// assert_eq!(s.offset(), 1);
-    /// s.next().unwrap();
-    /// assert_eq!(s.offset(), 2);
-    /// ```
-    fn offset(&self) -> usize;
-
-    /// Return a "checkpoint" that you can later pass to [`Tokens::rewind_to_checkpoint`]
+    /// Return a "location" that you can later pass to [`Tokens::rewind_to_location`]
     /// to reset the tokens back to the state at the time it was handed out.
     ///
     /// # Example
@@ -37,62 +23,165 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
     ///
     /// let mut s = "abcde".into_tokens();
     ///
-    /// let checkpoint = s.save_checkpoint();
+    /// let location = s.location();
     ///
     /// assert_eq!(s.next().unwrap(), 'a');
-    /// assert_eq!(s.offset(), 1);
+    /// assert_eq!(s.consumed().len(), 1);
     /// assert_eq!(s.next().unwrap(), 'b');
-    /// assert_eq!(s.offset(), 2);
+    /// assert_eq!(s.consumed().len(), 2);
     ///
-    /// s.rewind_to_checkpoint(checkpoint);
+    /// s.rewind_to_location(location);
     ///
     /// assert_eq!(s.next().unwrap(), 'a');
-    /// assert_eq!(s.offset(), 1);
+    /// assert_eq!(s.consumed().len(), 1);
     /// assert_eq!(s.next().unwrap(), 'b');
-    /// assert_eq!(s.offset(), 2);
+    /// assert_eq!(s.consumed().len(), 2);
     /// ```
-    fn save_checkpoint(&self) -> Self::CheckPoint;
+    fn location(&self) -> Self::Location;
 
-    /// Reset the tokens to the checkpoint provided. If you provide a checkpoint that
+    /// Reset the tokens to the location provided. If you provide a location that
     /// is in the future, expect that this could panic (implementation dependent).
     ///
-    /// See [`Tokens::save_checkpoint`].
-    fn rewind_to_checkpoint(&mut self, checkpoint: Self::CheckPoint);
+    /// See [`Tokens::location`].
+    fn rewind_to_location(&mut self, location: Self::Location);
 
-    /**
-    Expect a specific token to be next. If the token is not found, the iterator is not 
-    advanced.
+    /// Return true if the current cursor location matches the location given, or false
+    /// otherwise.
+    /// 
+    /// # Example
+    /// 
+    /// ```
+    /// use yap::{ Tokens, IntoTokens };
+    /// 
+    /// let mut s = "abc".into_tokens();
+    /// let location = s.location();
+    /// assert_eq!(s.is_at_location(location), true);
+    /// s.next();
+    /// assert_eq!(s.is_at_location(location), false);
+    /// s.rewind_to_location(location);
+    /// assert_eq!(s.is_at_location(location), true);
+    /// ```
+    fn is_at_location(&self, location: Self::Location) -> bool;
 
-    # Example
+    /// Get back the next item in the input without consuming it.
+    /// 
+    /// # Example
+    /// 
+    /// ```
+    /// use yap::{ Tokens, IntoTokens };
+    ///
+    /// let mut s = "abc".into_tokens();
+    /// assert_eq!(s.peek(), Some('a'));
+    /// assert_eq!(s.peek(), Some('a'));
+    /// ```
+    fn peek(&mut self) -> Option<Self::Item> {
+        let location = self.location();
+        let item = self.next();
+        self.rewind_to_location(location);
+        item
+    }
 
-    ```
-    use yap::{ Tokens, IntoTokens };
-
-    let mut s = "abc".into_tokens();
-    assert_eq!(s.token(&'a'), Some('a'));
-    assert_eq!(s.token(&'b'), Some('b'));
-    assert_eq!(s.token('z'), None);
-    assert_eq!(s.token('y'), None);
-    assert_eq!(s.token('c'), Some('c'));
-    ```
-    */
-    fn token<I>(&mut self, t: I) -> Option<Item> 
+    /// Expect a specific token to be next. If the token is not found, the iterator is not 
+    /// advanced.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use yap::{ Tokens, IntoTokens };
+    ///
+    /// let mut s = "abc".into_tokens();
+    /// assert_eq!(s.token(&'a'), true);
+    /// assert_eq!(s.token(&'b'), true);
+    /// assert_eq!(s.token('z'), false);
+    /// assert_eq!(s.token('y'), false);
+    /// assert_eq!(s.token('c'), true);
+    /// ```
+    fn token<I>(&mut self, t: I) -> bool
     where 
-        Item: PartialEq,
-        I: Borrow<Item>
+        Self::Item: PartialEq,
+        I: Borrow<Self::Item>
     {
-        let checkpoint = self.save_checkpoint();
+        let location = self.location();
         match self.next() {
-            Some(item) if &item == t.borrow() => {
-                Some(item)
-            },
+            Some(item) if &item == t.borrow() => true,
             _ => {
-                self.rewind_to_checkpoint(checkpoint);
-                None
+                self.rewind_to_location(location);
+                false
             }
         }
     }
 
+    /// Expect a specific set of tokens to be next. If the tokens are not found, the iterator is not 
+    /// advanced. Anything that implements `IntoIterator` with an `Item` type that can be borrowed to
+    /// produce `&Item` can be provided as an input to this.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use yap::{ Tokens, IntoTokens };
+    ///
+    /// let mut s = "abcdef".into_tokens();
+    /// 
+    /// assert_eq!(s.tokens("abc".chars()), true);
+    /// assert_eq!(s.remaining(), "def");
+    /// 
+    /// assert_eq!(s.tokens("de".chars()), true);
+    /// assert_eq!(s.remaining(), "f");
+    /// ```
+    fn tokens<It>(&mut self, ts: It) -> bool
+    where 
+        Self::Item: PartialEq,
+        It: IntoIterator,
+        It::Item: Borrow<Self::Item>
+    {
+        let location = self.location();
+        // `ts` comes first to avoid consuming an extra item from self before 
+        // realising that it's time to stop..
+        for (expected, actual) in ts.into_iter().zip(self.into_iter()) {
+            if &actual != expected.borrow() {
+                self.rewind_to_location(location);
+                return false;
+            }
+        }
+        true
+    }
+
+    /// Return the first token that matches the tokens provided, or None if none of them
+    /// match.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use yap::{ Tokens, IntoTokens };
+    ///
+    /// let mut s = "abcdef".into_tokens();
+    ///
+    /// assert_eq!(s.one_of_tokens("abc".chars()), Some('a'));
+    /// assert_eq!(s.one_of_tokens("abc".chars()), Some('b'));
+    /// assert_eq!(s.one_of_tokens("abc".chars()), Some('c'));
+    /// assert_eq!(s.one_of_tokens("abc".chars()), None);
+    /// assert_eq!(s.remaining(), "def");
+    /// ```
+    fn one_of_tokens<It>(&mut self, ts: It) -> Option<Self::Item>
+    where 
+        Self::Item: PartialEq,
+        It: IntoIterator,
+        It::Item: Borrow<Self::Item>
+    {
+        for expected in ts.into_iter() {
+            let location = self.location();
+            match self.next() {
+                Some(token) if &token == expected.borrow() => {
+                    return Some(token)
+                },
+                _ => {
+                    self.rewind_to_location(location);
+                }
+            }
+        }
+        None
+    }
+
     /// Parse 0 or more instances of some parser, returning all of the successfully parsed
     /// output and leaving any input that was not successfully parsed.
     ///
@@ -101,7 +190,7 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
     /// ```rust
     /// use yap::{ Tokens, IntoTokens };
     ///
-    /// fn parse_digit(mut tokens: impl Tokens<char>) -> Option<u32> {
+    /// fn parse_digit(mut tokens: impl Tokens<Item=char>) -> Option<u32> {
     ///     let c = tokens.next()?;
     ///     c.to_digit(10)
     /// }
@@ -110,20 +199,20 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
     /// let digits = s.many(|t| parse_digit(t));
     ///
     /// assert_eq!(digits, vec![1,2,3,4,5]);
-    /// assert_eq!(s.str_remaining(), "abcde");
+    /// assert_eq!(s.remaining(), "abcde");
     /// ```
     fn many<F, Output>(&mut self, mut parser: F) -> Vec<Output> 
     where F: FnMut(&mut Self) -> Option<Output>
     {
         let mut out = vec![];
         loop {
-            let pos = self.save_checkpoint();
+            let pos = self.location();
             if let Some(output) = parser(self) {
                 out.push(output);
             } else {
                 // The provided parser failed to produce more output,
                 // so rewind to before it and end.
-                self.rewind_to_checkpoint(pos);
+                self.rewind_to_location(pos);
                 break out;
             }
         }
@@ -138,7 +227,7 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
     /// ```rust
     /// use yap::{ Tokens, IntoTokens };
     ///
-    /// fn parse_digit(mut tokens: impl Tokens<char>) -> Option<u32> {
+    /// fn parse_digit(mut tokens: impl Tokens<Item=char>) -> Option<u32> {
     ///     let c = tokens.next()?;
     ///     c.to_digit(10)
     /// }
@@ -149,27 +238,27 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
     /// let mut s = "12345abcde".into_tokens();
     /// let digits = s.many1(|t| parse_digit(t).ok_or("aaah"));
     /// assert_eq!(digits, Ok(vec![1,2,3,4,5]));
-    /// assert_eq!(s.str_remaining(), "abcde");
+    /// assert_eq!(s.remaining(), "abcde");
     ///
     /// // No digits at all; this won't work! `many` would just return an
     /// // empty Vec, but `many1` gives us back the error we encountered.
     /// let mut s = "abcde".into_tokens();
     /// let digits = s.many1(|t| parse_digit(t).ok_or("aaah"));
     /// assert_eq!(digits, Err("aaah"));
-    /// assert_eq!(s.str_remaining(), "abcde");
+    /// assert_eq!(s.remaining(), "abcde");
     /// ```
     fn many1<F, E, Output>(&mut self, mut parser: F) -> Result<Vec<Output>, E> 
     where F: FnMut(&mut Self) -> Result<Output, E>
     {
         let mut out = vec![];
         loop {
-            let pos = self.save_checkpoint();
+            let pos = self.location();
             match parser(self) {
                 Ok(output) => {
                     out.push(output);
                 },
                 Err(e) => {
-                    self.rewind_to_checkpoint(pos);
+                    self.rewind_to_location(pos);
                     if out.is_empty() {
                         break Err(e)
                     } else {                    
@@ -188,7 +277,7 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
     /// use yap::{ Tokens, IntoTokens };
     ///
     /// struct ABC;
-    /// fn parse_abc(mut tokens: impl Tokens<char>) -> Option<ABC> {
+    /// fn parse_abc(mut tokens: impl Tokens<Item=char>) -> Option<ABC> {
     ///     let a = tokens.next()?;
     ///     let b = tokens.next()?;
     ///     let c = tokens.next()?;
@@ -202,17 +291,17 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
     /// let mut s = "abcabcababab".into_tokens();
     /// s.skip_many(|t| parse_abc(t).is_some());
     ///
-    /// assert_eq!(s.str_remaining(), "ababab");
+    /// assert_eq!(s.remaining(), "ababab");
     /// ```
     fn skip_many<F>(&mut self, mut parser: F)
     where F: FnMut(&mut Self) -> bool
     {
         loop {
-            let pos = self.save_checkpoint();
+            let pos = self.location();
             if !parser(self) {
                 // The provided parser failed to produce more output,
                 // so rewind to before it and end.
-                self.rewind_to_checkpoint(pos);
+                self.rewind_to_location(pos);
                 break;
             }
         }
@@ -227,7 +316,7 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
     /// use yap::{ Tokens, IntoTokens };
     ///
     /// struct ABC;
-    /// fn parse_abc(mut tokens: impl Tokens<char>) -> Option<ABC> {
+    /// fn parse_abc(mut tokens: impl Tokens<Item=char>) -> Option<ABC> {
     ///     let a = tokens.next()?;
     ///     let b = tokens.next()?;
     ///     let c = tokens.next()?;
@@ -242,16 +331,16 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
     /// let digits = s.skip_many1(|t| parse_abc(t).ok_or("aaah"));
     ///
     /// assert_eq!(digits, Err("aaah"));
-    /// assert_eq!(s.str_remaining(), "ababababcabc");
+    /// assert_eq!(s.remaining(), "ababababcabc");
     /// ```
     fn skip_many1<F, E, Ignored>(&mut self, mut parser: F) -> Result<(), E> 
     where F: FnMut(&mut Self) -> Result<Ignored, E>
     {
         let mut has_seen = false;
         loop {
-            let pos = self.save_checkpoint();
+            let pos = self.location();
             if let Err(e) = parser(self) {
-                self.rewind_to_checkpoint(pos);
+                self.rewind_to_location(pos);
                 if !has_seen {
                     break Err(e)
                 } else {                    
@@ -277,19 +366,18 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
     /// let mut s = "12345abc".into_tokens();
     /// let digits = s.take_tokens_while(|c| c.is_numeric());
     /// assert_eq!(digits, vec!['1','2','3','4','5']);
-    /// assert_eq!(s.str_remaining(), "abc");
+    /// assert_eq!(s.remaining(), "abc");
     /// ```
-    fn take_tokens_while<F>(&mut self, mut f: F) -> Vec<Item> 
-    where F: FnMut(&Item) -> bool
+    fn take_tokens_while<F>(&mut self, mut f: F) -> Vec<Self::Item> 
+    where F: FnMut(&Self::Item) -> bool
     {
         let mut toks = vec![];
         loop {
-            let pos = self.save_checkpoint();
-            if let Some(item) = self.next() {
-                if f(&item) {
-                    toks.push(item);
-                } else {
-                    self.rewind_to_checkpoint(pos);
+            let pos = self.location();
+            match self.next() {
+                Some(item) if f(&item) => toks.push(item),
+                _ => {
+                    self.rewind_to_location(pos);
                     break toks;
                 }
             }
@@ -307,10 +395,10 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
     /// let mut s = "abc12345".into_tokens();
     /// let digits = s.take_tokens_until(|c| c.is_numeric());
     /// assert_eq!(digits, vec!['a', 'b', 'c']);
-    /// assert_eq!(s.str_remaining(), "12345");
+    /// assert_eq!(s.remaining(), "12345");
     /// ```
-    fn take_tokens_until<F>(&mut self, mut f: F) -> Vec<Item> 
-    where F: FnMut(&Item) -> bool
+    fn take_tokens_until<F>(&mut self, mut f: F) -> Vec<Self::Item> 
+    where F: FnMut(&Self::Item) -> bool
     {
         self.take_tokens_while(|t| !f(t))
     }
@@ -325,18 +413,17 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
     ///
     /// let mut s = "12345abc".into_tokens();
     /// s.skip_tokens_while(|c| c.is_numeric());
-    /// assert_eq!(s.str_remaining(), "abc");
+    /// assert_eq!(s.remaining(), "abc");
     /// ```
     fn skip_tokens_while<F>(&mut self, mut f: F)
-    where F: FnMut(&Item) -> bool
+    where F: FnMut(&Self::Item) -> bool
     {
         loop {
-            let pos = self.save_checkpoint();
-            if let Some(item) = self.next() {
-                if f(&item) {
-                    // item found; keep going
-                } else {
-                    self.rewind_to_checkpoint(pos);
+            let pos = self.location();
+            match self.next() {
+                Some(item) if f(&item) => { /* item found; keep going */ }
+                _ => {
+                    self.rewind_to_location(pos);
                     break;
                 }
             }
@@ -353,10 +440,10 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
     ///
     /// let mut s = "abc12345".into_tokens();
     /// s.skip_tokens_until(|c| c.is_numeric());
-    /// assert_eq!(s.str_remaining(), "12345");
+    /// assert_eq!(s.remaining(), "12345");
     /// ```
     fn skip_tokens_until<F>(&mut self, mut f: F)
-    where F: FnMut(&Item) -> bool
+    where F: FnMut(&Self::Item) -> bool
     {
         self.skip_tokens_while(|t| !f(t))
     }
@@ -369,15 +456,15 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
     /// ```
     /// use yap::{ Tokens, IntoTokens };
     ///
-    /// fn parse_digit(mut tokens: impl Tokens<char>) -> Option<u32> {
+    /// fn parse_digit(mut tokens: impl Tokens<Item=char>) -> Option<u32> {
     ///     let c = tokens.next()?;
     ///     c.to_digit(10)
     /// }
     ///
     /// let mut s = "1,2,3,4,abc".into_tokens();
-    /// let digits = s.sep_by(|t| parse_digit(t), |t| t.token(',').is_some());
+    /// let digits = s.sep_by(|t| parse_digit(t), |t| t.token(','));
     /// assert_eq!(digits, vec![1,2,3,4]);
-    /// assert_eq!(s.str_remaining(), ",abc");
+    /// assert_eq!(s.remaining(), ",abc");
     /// ```
     fn sep_by<F, S, Output>(&mut self, mut parser: F, mut separator: S) -> Vec<Output> 
     where
@@ -385,24 +472,24 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
         S: FnMut(&mut Self) -> bool
     {
         let mut out = vec![];
-        let mut last_out_pos = self.save_checkpoint();
+        let mut last_out_pos = self.location();
         loop {
             match parser(self) {
                 Some(output) => {
                     out.push(output);
-                    last_out_pos = self.save_checkpoint();
+                    last_out_pos = self.location();
         
                     if !separator(self) {
                         // No separator? rewind to after the last output and return
                         // what we have so far.
-                        self.rewind_to_checkpoint(last_out_pos);
+                        self.rewind_to_location(last_out_pos);
                         break out;
                     }
                 }
                 None => {
                     // No output? We may have parsed a separator last time round!
                     // Revert to after last output and return what we have.
-                    self.rewind_to_checkpoint(last_out_pos);
+                    self.rewind_to_location(last_out_pos);
                     break out;
                 }
             }
@@ -418,15 +505,15 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
     /// ```
     /// use yap::{ Tokens, IntoTokens };
     ///
-    /// fn parse_digit(mut tokens: impl Tokens<char>) -> Option<u32> {
+    /// fn parse_digit(mut tokens: impl Tokens<Item=char>) -> Option<u32> {
     ///     let c = tokens.next()?;
     ///     c.to_digit(10)
     /// }
     ///
     /// let mut s = "a,1,2,3".into_tokens();
-    /// let digits = s.sep_by1(|t| parse_digit(t).ok_or("aah"), |t| t.token(',').is_some());
+    /// let digits = s.sep_by1(|t| parse_digit(t).ok_or("aah"), |t| t.token(','));
     /// assert_eq!(digits, Err("aah"));
-    /// assert_eq!(s.str_remaining(), "a,1,2,3");
+    /// assert_eq!(s.remaining(), "a,1,2,3");
     /// ```
     fn sep_by1<F, S, E, Output>(&mut self, mut parser: F, mut separator: S) -> Result<Vec<Output>, E> 
     where
@@ -434,24 +521,24 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
         S: FnMut(&mut Self) -> bool
     {
         let mut out = vec![];
-        let mut last_out_pos = self.save_checkpoint();
+        let mut last_out_pos = self.location();
         loop {
             match parser(self) {
                 Ok(output) => {
                     out.push(output);
-                    last_out_pos = self.save_checkpoint();
+                    last_out_pos = self.location();
             
                     if !separator(self) {
                         // No separator? rewind to after the last output and return
                         // what we have so far.
-                        self.rewind_to_checkpoint(last_out_pos);
+                        self.rewind_to_location(last_out_pos);
                         break Ok(out);
                     }
                 },
                 Err(e) => {
                     // Reset to last output parsed. Return error if no output 
                     // was successfully parsed at all.
-                    self.rewind_to_checkpoint(last_out_pos);
+                    self.rewind_to_location(last_out_pos);
                     break if out.is_empty() { Err(e) } else { Ok(out) };
                 }
             }
@@ -471,7 +558,7 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
     /// #[derive(PartialEq,Debug)]
     /// enum OpOrDigit { Op(Op), Digit(u32) }
     ///
-    /// fn parse_op(mut t: impl Tokens<char>) -> Option<Op> {
+    /// fn parse_op(mut t: impl Tokens<Item=char>) -> Option<Op> {
     ///     match t.next()? {
     ///         '-' => Some(Op::Minus),
     ///         '+' => Some(Op::Plus),
@@ -480,7 +567,7 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
     ///     }
     /// }
     ///
-    /// fn parse_digit(mut tokens: impl Tokens<char>) -> Option<u32> {
+    /// fn parse_digit(mut tokens: impl Tokens<Item=char>) -> Option<u32> {
     ///     let c = tokens.next()?;
     ///     c.to_digit(10)
     /// }
@@ -499,7 +586,7 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
     ///     OpOrDigit::Op(Op::Minus),
     ///     OpOrDigit::Digit(4),
     /// ]);
-    /// assert_eq!(s.str_remaining(), "+abc");
+    /// assert_eq!(s.remaining(), "+abc");
     /// ```
     fn sep_by_all<F, S, Output>(&mut self, mut parser: F, mut separator: S) -> Vec<Output> 
     where
@@ -507,12 +594,12 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
         S: FnMut(&mut Self) -> Option<Output>
     {
         let mut out = vec![];
-        let mut last_out_pos = self.save_checkpoint();
+        let mut last_out_pos = self.location();
         loop {
             match parser(self) {
                 Some(output) => {
                     out.push(output);
-                    last_out_pos = self.save_checkpoint();
+                    last_out_pos = self.location();
         
                     match separator(self) {
                         Some(sep) => {
@@ -522,7 +609,7 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
                         None => {
                             // No separator? rewind to after the last output and return
                             // what we have so far.
-                            self.rewind_to_checkpoint(last_out_pos);
+                            self.rewind_to_location(last_out_pos);
                             break out;
                         }
                     }
@@ -531,7 +618,7 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
                     // No output? We're either just starting, or we just parsed a separator,
                     // so rewind and throw away the mis-parsed separator (if there is one).
                     out.pop();
-                    self.rewind_to_checkpoint(last_out_pos);
+                    self.rewind_to_location(last_out_pos);
                     break out;
                 }
             }
@@ -546,12 +633,12 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
         S: FnMut(&mut Self) -> Option<Output>
     {
         let mut out = vec![];
-        let mut last_out_pos = self.save_checkpoint();
+        let mut last_out_pos = self.location();
         loop {
             match parser(self) {
                 Ok(output) => {
                     out.push(output);
-                    last_out_pos = self.save_checkpoint();
+                    last_out_pos = self.location();
             
                     match separator(self) {
                         Some(sep) => {
@@ -561,7 +648,7 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
                         None => {
                             // No separator? rewind to after the last output and return
                             // what we have so far.
-                            self.rewind_to_checkpoint(last_out_pos);
+                            self.rewind_to_location(last_out_pos);
                             break Ok(out);
                         }
                     }
@@ -569,35 +656,160 @@ pub trait Tokens<Item>: Iterator<Item=Item> {
                 Err(e) => {
                     // Reset to last output parsed. Return error if no output 
                     // was successfully parsed at all.
-                    self.rewind_to_checkpoint(last_out_pos);
+                    self.rewind_to_location(last_out_pos);
                     break if out.is_empty() { Err(e) } else { Ok(out) };
                 }
             }
         }
     }
 
+    /// Parse some tokens optionally surrounded by the tokens consumed by the surrounding parser.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use yap::{ Tokens, IntoTokens };
+    ///
+    /// let mut s = "   hello    ".into_tokens();
+    ///
+    /// let hello: String = s.surrounded_by(
+    ///     |t| t.take_tokens_while(|c| c.is_ascii_alphabetic()).into_iter().collect(),
+    ///     |t| t.skip_tokens_while(|c| c.is_ascii_whitespace())
+    /// );
+    ///
+    /// assert_eq!(&*hello, "hello");
+    /// assert_eq!(s.remaining(), "");
+    /// ```
+    fn surrounded_by<F, S, Output>(&mut self, mut parser: F, mut surrounding: S) -> Output 
+    where
+        F: FnMut(&mut Self) -> Output,
+        S: FnMut(&mut Self)
+    {
+        self.skip_optional(&mut surrounding);
+        let res = parser(self);
+        self.skip_optional(&mut surrounding);
+        res
+    }
+
+    /// Attempt to parse some output from the tokens. If the function returns `None`,
+    /// don't consume any input. Else, return whatever the function produced.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use yap::{ Tokens, IntoTokens };
+    ///
+    /// let mut s = "foobar".into_tokens();
+    ///
+    /// let res = s.optional(|s| {
+    ///     let a = s.next();
+    ///     let b = s.next();
+    ///     if a == b {
+    ///         Some("yay")
+    ///     } else {
+    ///         None
+    ///     }
+    /// });
+    ///
+    /// // nothing consumed since None returned from fn
+    /// assert_eq!(s.remaining(), "foobar");
+    /// assert_eq!(res, None);
+    ///
+    /// let res = s.optional(|s| {
+    ///     let a = s.next()?;
+    ///     let b = s.next()?;
+    ///     Some((a, b))
+    /// });
+    ///
+    /// // 2 chars consumed since Some returned from fn
+    /// assert_eq!(s.remaining(), "obar");
+    /// assert_eq!(res, Some(('f', 'o')));
+    /// ```
+    fn optional<F, Output>(&mut self, mut f: F) -> Option<Output> 
+    where F: FnMut(&mut Self) -> Option<Output> {
+        let location = self.location();
+        match f(self) {
+            Some(output) => Some(output),
+            None => {
+                self.rewind_to_location(location);
+                None
+            }
+        }
+    }
+
+    /// Run a parser against some tokens, and don't care whether it succeeded
+    /// or how much input it consumed.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use yap::{ Tokens, IntoTokens };
+    ///
+    /// let mut s = "   helloworld".into_tokens();
+    ///
+    /// fn parse_whitespace(mut t: impl Tokens<Item=char>) {
+    ///     t.skip_tokens_while(|c| c.is_ascii_whitespace());
+    /// }
+    ///
+    /// s.skip_optional(|t| parse_whitespace(t));
+    /// let is_hello = s.tokens("hello".chars());
+    /// s.skip_optional(|t| parse_whitespace(t));
+    /// let world: String = s.take_tokens_while(|c| c.is_ascii_alphabetic()).into_iter().collect();
+    ///
+    /// // assert_eq!(is_hello, true);
+    /// // assert_eq!(&*world, "world");
+    /// ```
+    fn skip_optional<F>(&mut self, mut f: F)
+    where F: FnMut(&mut Self) {
+        self.optional(|t| {
+            f(t);
+            Some(())
+        });
+    }
+
 }
 
-impl <'a, Item, T> Tokens<Item> for &'a mut T 
-where T: Tokens<Item>
-{
-    type CheckPoint = T::CheckPoint;
+pub trait TokenLocation {
+    /// Return the current offset into the tokens that we've parsed up to so far.
+    /// The exact meaning of this can vary by implementation; when parsing slices, it
+    /// is index of the slice item we've consumed up to (it may equal `slice.len()`), and when
+    /// parsing `&str`'s it is the number of bytes (not characters) consumed so far.
+    /// 
+    /// # Example
+    /// 
+    /// ```
+    /// use yap::{ Tokens, IntoTokens, TokenLocation };
+    /// 
+    /// let mut s = "abc".into_tokens();
+    /// assert_eq!(s.location().offset(), 0);
+    /// s.next();
+    /// assert_eq!(s.location().offset(), 1);
+    /// s.next();
+    /// assert_eq!(s.location().offset(), 2);
+    /// ```
+    fn offset(&self) -> usize;
+}
 
-    fn offset(&self) -> usize {
-        <T as Tokens<Item>>::offset(self)
+impl <'a, T> Tokens for &'a mut T 
+where T: Tokens
+{
+    type Location = T::Location;
+
+    fn location(&self) -> Self::Location {
+        <T as Tokens>::location(self)
     }
-    fn save_checkpoint(&self) -> Self::CheckPoint {
-        <T as Tokens<Item>>::save_checkpoint(self)
+    fn rewind_to_location(&mut self, location: Self::Location) {
+        <T as Tokens>::rewind_to_location(self, location)
     }
-    fn rewind_to_checkpoint(&mut self, checkpoint: Self::CheckPoint) {
-        <T as Tokens<Item>>::rewind_to_checkpoint(self, checkpoint)
+    fn is_at_location(&self, location: Self::Location) -> bool {
+        <T as Tokens>::is_at_location(self, location)
     }
 }
 
 /// A trait that is implemented by anything which can be converted into an 
 /// object implementing the [`Tokens`] trait.
 pub trait IntoTokens<Item> {
-    type Tokens: Tokens<Item>;
+    type Tokens: Tokens<Item=Item>;
     fn into_tokens(self) -> Self::Tokens;
 }
 
@@ -614,7 +826,7 @@ mod test {
     // A simple parser that looks for "ab" in an input token stream.
     // Notably, it doesn't try to rewind on failure. We expect the *many*
     // combinators to take care of that sort of thing for us as needed.
-    fn parse_ab(mut t: impl Tokens<char>) -> Option<AB> {
+    fn parse_ab(mut t: impl Tokens<Item=char>) -> Option<AB> {
         // match any sequence "ab".
         let a = t.next()?;
         let b = t.next()?;
